"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.tryGetSecretValue = exports.constructVaultName = exports.ensureRequireCloudResourcesForPulumiStateExist = void 0;
const http = __importStar(require("@azure/core-http"));
const resources = __importStar(require("@azure/arm-resources"));
const storage = __importStar(require("@azure/arm-storage"));
const kv = __importStar(require("@azure/arm-keyvault-profile-2020-09-01-hybrid"));
const secrets = __importStar(require("@azure/keyvault-secrets"));
const keys = __importStar(require("@azure/keyvault-keys"));
const secretGetting = __importStar(require("@data-heaving/azure-kv-secret"));
const common = __importStar(require("@data-heaving/common"));
const utils = __importStar(require("./run-common"));
const ensureRequireCloudResourcesForPulumiStateExist = async (inputs) => {
    const { eventEmitter, organization } = inputs;
    const clientArgs = [inputs.credentials, inputs.azure.subscriptionId];
    // Upsert resource group
    const cicdRGName = await ensureResourceGroupExists(eventEmitter, clientArgs, organization);
    const storageContainerName = "bootstrap";
    const [{ vaultName, encryptionKeyURL }, storageAccountInfo] = await Promise.all([
        ensureKeyVaultIsConfigured(clientArgs, inputs, cicdRGName, storageContainerName),
        ensureStorageAccountIsConfigured(eventEmitter, clientArgs, organization, cicdRGName, storageContainerName),
    ]);
    return {
        cicdRGName,
        kvName: vaultName,
        backendConfig: {
            storageAccountName: storageAccountInfo.storageAccountName,
            storageContainerName,
            encryptionKeyURL,
        },
        backendStorageAccountKey: storageAccountInfo.storageAccountKey,
    };
};
exports.ensureRequireCloudResourcesForPulumiStateExist = ensureRequireCloudResourcesForPulumiStateExist;
const ensureResourceGroupExists = async (eventEmitter, clientArgs, { name: organization, location }) => {
    const rg = await new resources.ResourceManagementClient(...clientArgs).resourceGroups.createOrUpdate(`${organization}-cicd`, { location });
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const rgName = rg.name;
    eventEmitter.emit("resourceGroupCreatedOrUpdated", rg);
    return rgName;
};
const ensureStorageAccountIsConfigured = async (eventEmitter, clientArgs, { name: organization, location }, rgName, containerName) => {
    const { storageAccounts, blobServices, blobContainers } = new storage.StorageManagementClient(...clientArgs);
    const updateArgs = {
        accessTier: "Cool",
        allowBlobPublicAccess: false,
        allowSharedKeyAccess: true,
        enableHttpsTrafficOnly: true,
        minimumTlsVersion: "TLS1_2",
    };
    const saName = `${organization.replace(/[-_]/g, "")}cicd`;
    eventEmitter.emit("storageAccountCreatedOrUpdated", await ((await storageAccounts.checkNameAvailability(saName)).nameAvailable === true
        ? storageAccounts.create(rgName, saName, {
            ...updateArgs,
            location,
            kind: "StorageV2",
            sku: {
                name: "Standard_GRS",
            },
        })
        : storageAccounts.update(rgName, saName, updateArgs)));
    await Promise.all([
        // Enable blob versioning for SA
        (async () => eventEmitter.emit("storageAccountBlobServicesConfigured", await blobServices.setServiceProperties(rgName, saName, {
            isVersioningEnabled: true,
        })))(),
        // Upsert container for Pulumi state in SA (notice that this time just using 'create' is enough, unlike with storage account)
        (async () => eventEmitter.emit("storageAccountContainerCreatedOrUpdated", await blobContainers.create(rgName, saName, containerName, {
            publicAccess: "None",
        })))(),
    ]);
    return {
        storageAccountName: saName,
        storageAccountKey: (await storageAccounts.listKeys(rgName, saName)).keys?.[0]?.value ?? "",
    };
};
const ensureKeyVaultIsConfigured = async (clientArgs, { eventEmitter, organization: { name: organization, location }, azure: { tenantId }, principalId, spAuthStorageConfig, pulumiEncryptionKeyBits, }, rgName, keyName) => {
    const { vaults } = new kv.KeyVaultManagementClient(...clientArgs);
    const vaultName = (0, exports.constructVaultName)(organization);
    const kvResult = await vaults.createOrUpdate(rgName, vaultName, {
        location,
        properties: {
            sku: {
                name: "standard",
            },
            tenantId,
            enabledForDeployment: false,
            enabledForDiskEncryption: false,
            enabledForTemplateDeployment: false,
            enableRbacAuthorization: true,
            createMode: "recover", // In order to recover previously deleted soft-deletable vaults
        },
    });
    const { id, properties: { vaultUri }, } = kvResult;
    eventEmitter.emit("keyVaultCreatedOrUpdated", kvResult);
    const kvURL = vaultUri; // eslint-disable-line @typescript-eslint/no-non-null-assertion
    const vaultID = id; // eslint-disable-line @typescript-eslint/no-non-null-assertion
    // Enable managing key vault for this SP
    eventEmitter.emit("keyVaultAdminRoleAssignmentCreatedOrUpdated", await utils.upsertRoleAssignment(clientArgs, vaultID, 
    // From https://docs.microsoft.com/en-us/azure/role-based-access-control/built-in-roles
    "00482a5a-887f-4fb3-b363-3b7fe8e74483", // "Key Vault Administrator"
    principalId));
    // The "getKeys" method exists only for "KeyVaultClient" class of "@azure/keyvault-keys" module.
    // However, the class is not exported -> therefore we have to make this ugly hack
    const keyClient = new keys.KeyClient(kvURL, clientArgs[0]);
    let key;
    try {
        key = await retryIf403(() => keyClient.getKey(keyName), "Waiting for key vault role assignment to propagate for encryption key...");
    }
    catch (e) {
        if (!(e instanceof http.RestError && e.code === "KeyNotFound")) {
            throw e;
        }
    }
    {
        let createNew = false;
        if (!key) {
            key = await keyClient.createRsaKey(keyName, {
                keySize: pulumiEncryptionKeyBits,
            });
            createNew = true;
        }
        eventEmitter.emit("keyCreatedOrUpdated", {
            createNew,
            key,
        });
    }
    const encryptionKeyURL = key.id; // eslint-disable-line @typescript-eslint/no-non-null-assertion
    // Store key + cert pem, if specified
    if (spAuthStorageConfig) {
        const { configSecretName: secretName } = spAuthStorageConfig;
        eventEmitter.emit("keyVaultAuthenticationSecretRoleAssignmentCreatedOrUpdated", await utils.upsertRoleAssignment(clientArgs, `${vaultID}/secrets/${secretName}`, 
        // From https://docs.microsoft.com/en-us/azure/role-based-access-control/built-in-roles
        "4633458b-17de-408a-b874-0445c86b69e6", // "Key Vault Secrets User",
        spAuthStorageConfig.configReaderPrincipalId));
        const existingSecretValue = await retryIf403(() => (0, exports.tryGetSecretValue)(clientArgs[0], {
            kvURL,
            secretName,
        }), "Waiting for key vault role assignment to propagate for bootstrapper app auth secret...");
        const secretValue = `${spAuthStorageConfig.keyPEM}${spAuthStorageConfig.certPEM}`;
        const createNew = existingSecretValue !== secretValue;
        if (createNew) {
            await new secrets.SecretClient(kvURL, clientArgs[0]).setSecret(secretName, secretValue);
        }
        eventEmitter.emit("authenticationSecretCreatedOrUpdated", {
            createNew,
            secretName,
            secretValue,
        });
    }
    return {
        vaultName,
        encryptionKeyURL, // eslint-disable-line @typescript-eslint/no-non-null-assertion
    };
};
const retryIf403 = async (getAction, message) => {
    let tryAgain = false;
    let retVal;
    do {
        try {
            retVal = await getAction();
            tryAgain = false;
        }
        catch (e) {
            if (e instanceof http.RestError && e.code === "Forbidden") {
                tryAgain = true;
            }
            else {
                throw e;
            }
        }
        if (tryAgain) {
            // eslint-disable-next-line no-console
            console.info(message);
            await common.sleep(10 * 1000);
        }
    } while (tryAgain);
    return retVal;
};
const constructVaultName = (organization) => `${organization}-cicd`;
exports.constructVaultName = constructVaultName;
const tryGetSecretValue = async (...args) => {
    let secretValue;
    try {
        secretValue = await secretGetting.getSecretValue(...args);
    }
    catch (e) {
        if (!isSecretNotFoundError(e)) {
            throw e;
        }
    }
    return secretValue;
};
exports.tryGetSecretValue = tryGetSecretValue;
const isSecretNotFoundError = (error) => error instanceof secretGetting.SecretDoesNotExistError ||
    (error instanceof http.RestError && error.code === "SecretNotFound");
//# sourceMappingURL=run-pulumi.js.map