"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BEGIN_CERTIFICATE = exports.tryGetSecretValue = exports.constructVaultName = exports.performBootstrap = void 0;
require("cross-fetch");
require("cross-fetch/polyfill");
const graph = __importStar(require("@microsoft/microsoft-graph-client"));
const id = __importStar(require("@azure/identity"));
const common = __importStar(require("@data-heaving/common"));
const fs = __importStar(require("fs/promises"));
const ad = __importStar(require("./run-ad"));
const auth = __importStar(require("./run-auth"));
const pulumiSetup = __importStar(require("./run-pulumi"));
const certs = __importStar(require("./run-certs"));
const performBootstrap = async (inputs) => {
    const bootstrapperInfo = await setupBootstrapperApp(inputs);
    const { backendConfig, backendStorageAccountKey, cicdRGName, kvName } = await runWithBootstrapper(inputs, bootstrapperInfo);
    const { bootstrapperPulumiAuth } = bootstrapperInfo;
    if (bootstrapperPulumiAuth.type === "sp") {
        bootstrapperPulumiAuth.backendStorageAccountKey = backendStorageAccountKey;
    }
    return {
        cicdRGName,
        kvName,
        pulumiConfigInfo: {
            backendConfig,
            auth: bootstrapperPulumiAuth,
        },
        envSpecificPipelineConfigReader: bootstrapperInfo.envSpecificPipelineConfigReader,
    };
};
exports.performBootstrap = performBootstrap;
const setupBootstrapperApp = async ({ eventEmitter, credentials: { credentials, givenClientId }, bootstrapperApp, azure: { tenantId, subscriptionId }, }) => {
    // 1. Ensure bootstrap SP is created using current Azure CLI credentials
    let clientId;
    let principalId;
    let bootstrapperCredentials;
    let bootstrapperPulumiAuth;
    let spAuthStorageConfig;
    const graphClient = graph.Client.initWithMiddleware({
        authProvider: credentials,
        debugLogging: true, // This will print URL of every command
    });
    const envSpecificPipelineConfigReader = bootstrapperApp.type === "msi"
        ? {
            principalId: bootstrapperApp.principalId,
            principalType: "ServicePrincipal", // It appears that in Azure role assignments, even MSIs will get this principal type instead of "MSI",
        }
        : await ad.getCurrentPrincipalInfo(graphClient, givenClientId);
    eventEmitter.emit("afterResolvingConfigReaderPrincipal", common.deepCopy(envSpecificPipelineConfigReader));
    switch (bootstrapperApp.type) {
        case "sp":
            {
                const { keyPath, certPath, rsaBits, pfxPath, pfxPassword, ...spAuth } = bootstrapperApp.authentication;
                await certs.ensureKeyAndCertExists(keyPath, certPath, rsaBits, spAuth.certValidityPeriodDays, spAuth.certSubject);
                const certPEM = certs.ensureEndsWithNewline(await fs.readFile(certPath, "utf-8"));
                const clientAndPrincipalID = await ad.ensureBootstrapSPIsConfigured({
                    eventEmitter,
                    graphClient,
                    bootstrapperApp,
                    certificatePEM: certPEM,
                });
                ({ clientId, principalId } = clientAndPrincipalID);
                const { keyPEM, keyAndCertPath } = await certs.ensureCertificateCredentialsFileExists(spAuth.tempDir, keyPath, certPEM);
                spAuthStorageConfig = {
                    keyPEM,
                    certPEM,
                    configReaderPrincipalId: envSpecificPipelineConfigReader.principalId,
                    configSecretName: bootstrapperApp.configSecretName,
                };
                bootstrapperCredentials = new id.ClientCertificateCredential(tenantId, clientId, keyAndCertPath);
                await certs.ensurePfxExists(keyPath, certPath, pfxPath, pfxPassword);
                bootstrapperPulumiAuth = {
                    type: "sp",
                    clientId,
                    pfxPath,
                    pfxPassword,
                    backendStorageAccountKey: "", // Will be modified by performSetUp
                };
            }
            break;
        case "msi":
            {
                ({ clientId, principalId } = bootstrapperApp);
                bootstrapperCredentials = new id.ManagedIdentityCredential(clientId);
                bootstrapperPulumiAuth = {
                    type: "msi",
                    clientId,
                };
            }
            break;
        default:
            throw new Error(`Unrecognized bootstrapper app kind: "${bootstrapperApp.type}"`);
    }
    // 2. Ensure bootstrap SP has enough permissions to continue operating
    await auth.ensureBootstrapSPHasEnoughPrivileges({
        eventEmitter,
        credentials,
        principalId,
        subscriptionId,
    });
    return {
        bootstrapperCredentials,
        bootstrapperPulumiAuth,
        principalId,
        spAuthStorageConfig,
        envSpecificPipelineConfigReader,
    };
};
const runWithBootstrapper = async ({ eventEmitter, azure, organization, pulumiEncryptionKeyBits }, { bootstrapperCredentials, principalId, spAuthStorageConfig, }) => {
    // 4. Create credentials for bootstrap SP, and then create necessary resources for Pulumi state management using bootstrap SP
    return await pulumiSetup.ensureRequireCloudResourcesForPulumiStateExist({
        eventEmitter,
        credentials: bootstrapperCredentials,
        azure,
        organization,
        principalId,
        spAuthStorageConfig,
        pulumiEncryptionKeyBits,
    });
};
exports.constructVaultName = pulumiSetup.constructVaultName;
exports.tryGetSecretValue = pulumiSetup.tryGetSecretValue;
exports.BEGIN_CERTIFICATE = certs.BEGIN_CERTIFICATE;
//# sourceMappingURL=run.js.map