import * as authArm from "@azure/arm-authorization";
import * as t from "io-ts";
import * as validation from "@data-heaving/common-validation";
import * as uuid from "uuid";
export const listOf = (item) => t.array(item, `${item.name}List`);
export const upsertRoleAssignment = async (clientOrArgs, scope, roleDefinitionUuId, principalId) => {
    const client = Array.isArray(clientOrArgs)
        ? new authArm.AuthorizationManagementClient(...clientOrArgs)
        : clientOrArgs;
    const roleDefinitionId = `/subscriptions/${client.subscriptionId}/providers/Microsoft.Authorization/roleDefinitions/${roleDefinitionUuId}`;
    const { roleAssignments } = client;
    return (validation
        .decodeOrThrow(listOf(roleAssignment).decode, await roleAssignments.listForScope(scope, {
        // Notice: if you re-run this code quickly in succession, bear in mind that this filter does not immediately see role assignment created on previous run.
        // This will result in "RoleAssignmentExists" error.
        filter: `principalId eq '${principalId}'`, // It would be nice to have " and atScope()" at the end but unfortunately, not supported. I think there will be less results this way as opposed to just atScope()
    }))
        // The array contains also role assignments for anything within the subscription
        .filter((assignment) => assignment.scope === scope &&
        assignment.roleDefinitionId === roleDefinitionId)[0] ??
        (await roleAssignments.create(scope, uuid.v4(), {
            principalId,
            roleDefinitionId,
        })));
};
const roleAssignment = t.type({
    type: t.literal("Microsoft.Authorization/roleAssignments"),
    id: validation.nonEmptyString,
    name: validation.uuid,
    scope: validation.nonEmptyString,
    roleDefinitionId: validation.nonEmptyString,
    principalId: validation.uuid,
    principalType: t.union([
        t.literal("User"),
        t.literal("Group"),
        t.literal("ServicePrincipal"),
        t.literal("Unknown"),
        t.literal("DirectoryRoleTemplate"),
        t.literal("ForeignGroup"),
        t.literal("Application"),
        t.literal("MSI"),
        t.literal("DirectoryObjectOrGroup"),
        t.literal("Everyone"),
    ], "PrincipalType"),
}, "RoleAssignment");
//# sourceMappingURL=run-common.js.map