import * as common from "@data-heaving/common";
import * as pipeline from "@data-heaving/pulumi-azure-pipeline";
import * as pulumiSetup from "@data-heaving/pulumi-azure-pipeline-setup";
import * as bootstrap from "./bootstrap";
const DEFAULT_STORAGE_CONTAINER_PREFIX = "cicd-env-";
const DEFAULT_KEY_NAME_PREFIX = "cicd-env-";
const DEFAULT_SECRET_NAME_PREFIX = "cicd-env-";
export const main = async ({ eventEmitters, doChanges, credentials, bootstrapperApp, organization, azure, targetResources, namingConventions, pipelineConfigs, }) => {
    const { bootstrapEventEmitter, pipelineEventEmitter, pulumiEventEmitters } = eventEmitters;
    const { cicdRGName, kvName, pulumiConfigInfo, envSpecificPipelineConfigReader, } = await bootstrap.performBootstrap({
        eventEmitter: bootstrapEventEmitter,
        credentials,
        azure,
        organization,
        pulumiEncryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForBootstrapper,
        bootstrapperApp: getBootstrapAppForSetup(bootstrapperApp, organization),
    });
    pipelineEventEmitter?.emit("beforeRunningPulumiPortion", {
        organization: common.deepCopy(organization),
        doChanges,
    });
    // For some reason, due some TS compiler bug, we must specify generic argument explicitly.
    const command = doChanges ? "up" : "preview";
    return await pipeline.runPulumiPipeline({
        pulumi: {
            auth: pulumiConfigInfo.auth,
            backendConfig: pulumiConfigInfo.backendConfig,
            programArgs: {
                projectName: "bootstrap",
                stackName: "main",
                program: () => pulumiSetup.pulumiProgram({
                    organization,
                    envSpecificPipelineConfigReader,
                    pulumiPipelineConfig: {
                        auth: bootstrapperApp.type === "sp" &&
                            bootstrapperApp.envSpecificPulumiPipelineSPAuth
                            ? {
                                type: "sp",
                                certificateConfig: bootstrapperApp.envSpecificPulumiPipelineSPAuth,
                            }
                            : {
                                type: "msi",
                                sharedSARGName: cicdRGName,
                                sharedSAName: pulumiConfigInfo.backendConfig.storageAccountName,
                                containerPrefixString: namingConventions?.storageContainerPrefixString ??
                                    DEFAULT_STORAGE_CONTAINER_PREFIX,
                            },
                        pulumiKVInfo: {
                            rgName: cicdRGName,
                            name: kvName,
                            keyNamePrefix: namingConventions?.keyNamePrefix ?? DEFAULT_KEY_NAME_PREFIX,
                            secretNamePrefix: namingConventions?.secretNamePrefix ??
                                DEFAULT_SECRET_NAME_PREFIX,
                            encryptionKeyBits: pipelineConfigs.pulumiEncryptionKeyBitsForEnvSpecificPipeline,
                        },
                    },
                    targetResources,
                }),
            },
        },
        azure: {
            tenantId: azure.tenantId,
            // Intentionally leave out subscription ID in order to cause errors if default provider is accidentally used.
            subscriptionId: undefined,
        },
    }, ["azure-native", "azuread", "tls"], command, pulumiEventEmitters);
};
const getBootstrapAppForSetup = (app, organization) => app.type === "msi"
    ? app
    : {
        ...app,
        willNeedToCreateAADApps: app.envSpecificPulumiPipelineSPAuth !== undefined &&
            organization.environments.length > 0,
    };
//# sourceMappingURL=pipeline.js.map